# pict-rs 0.5.18

pict-rs is a simple image hosting microservice, designed to handle storing and retrieving images,
animations, and videos, as well as providing basic image processing functionality.


## Overview

pict-rs 0.5.18 adds a new repo backend called Fjall. This is likely the future default backend for
pict-rs when 0.6 releases. Otherwise, there are a few bug fixes and the README has been updated with
more information about API responses.


## Upgrade Notes

### Headline Features

- [Fjall Repo](#fjall-repo)


### Fixes

- [Last Hashes in Repo Migration](#last-hashes-in-repo-migration)
- [Alias Access in Repo Migration](#alias-access-in-repo-migration)
- [Hash Variant Cleanup](#variant-cleanup)
- [Duplicate Upload Details](#duplicate-upload-details)


### Changes

- [Update Readme](#update-readme)


### Features

- [Add Tag to Health Check File](#add-tag-to-health-check-file)


## Details

### Fjall Repo

A new repo has been added to pict-rs, another embedded database like sled called
[`fjall`](https://fjall-rs.github.io/). The primary reason for this is that sled 0.34.7 has proven
to contain a few (rare) bugs that have been annoying some admins (see
[#79](https://git.asonix.dog/asonix/pict-rs/issues/79)). There is a complete rewrite of Sled
currently in progress, but I had expected it to be complete upwards of a year ago. While it seems
like the release of sled 1.0 is much closer now, there's still no indication of exactly when that
might be.

In the process of adding a new embedded repo, I also tried to plan for more easy debugging. To that
end, the contents of the fjall repo can be dumped in a debug representation to a text file on launch
to help admins get a sense of what the state of the database is. This file will be very large for
real production pict-rs deployments, so it shouldn't be used frequently.

In the future, more debugging tools can be created based on this.

In order to make use of the fjall repo, you can migrate using the pict-rs migrate-repo command, and
then the pict-rs server can be configured in all the usual ways.

Migration Example:
```bash
pict-rs migrate-repo \
    sled \
        -p ./data/sled-repo-local \
    fjall \
        -f ./data/fjall-repo-local
```

Configuration Examples:
```bash
# with environment variables
PICTRS__REPO__TYPE=fjall
PICTRS__REPO__FJALL_PATH=/mnt/fjall-repo
```

```toml
# pict-rs.toml
[repo]
type = 'fjall'
fjall_path = '/mnt/fjall-repo'
```

```bash
# commandline
pict-rs run \
    filesystem \
        -p /mnt/files \
    fjall \
        -f /mnt/fjall-repo
```


### Last Hashes in Repo Migration

Since the introduction of repo migration concurrency in 0.5.17, it was possible for the migration
process to exit before all hashes had been fully migrated. The number of potentially unmigrated
hashes corresponds directly with the concurrency limit used during the migration. The default limit
of 32 implies that up to 32 hashes could have failed to migrate.

pict-rs now properly waits for all hashes to finish migrating before ending the process.


### Alias Access in Repo Migration

When migrating repos from sled, previously pict-rs failed to keep proper track of alias accesses.
This would lead to all proxied media having last access timestamps of the time of migration, rather
than the actual time they were last accessed. While this wouldn't cause any user-noticeable
problems, it could cause heavier load after the proxy retention period expires and all the proxied
media would get queued for cleanup at the same time.

This is now fixed and future migrations will properly assign alias accesses.


### Hash Variant Cleanup

When removing files from pict-rs, previously if all aliases of a file had been deleted, it was
possible for details records for deleted files to be left in the repo. This should no longer occur
in 0.5.18. There is no included cleanup for existing orphaned details records.


### Duplicate Upload Details

When uploading a duplicate file to pict-rs, a details record would be inserted for the duplicate
file and wouldn't get removed after duplicate detection was applied. pict-rs 0.5.18 no longer
inserts these details records.


### Update Readme

Previous releases had failed to update the API docs in the readme. I've now gone & added the
relevant changes as well as more information about HTTP response codes. I generally try to make the
response codes that pict-rs uses make sense :)

### Add Tag to Health Check File

pict-rs 0.5.18 adds a new configuration option for object storage to enable setting a tag on the
`health-check` file created when using the `/healthz` endpoint. This can be useful when targetting
the health-check file with rules in some systems. Since not all object storage implementations
support tags, this is not enabled by default.

See `[pict-rs.toml](https://git.asonix.dog/asonix/pict-rs/src/branch/main/pict-rs.toml)` for
configuration information.
