use crate::path_gen::{PathItem, PathNode};
use std::{borrow::Cow, sync::Arc};

use super::FjallError;

pub(super) struct Blurhash(pub Arc<str>);

impl<'a> PathItem<'a> for Blurhash {
    type Error = FjallError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        Cow::Borrowed(self.0.as_bytes())
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        match bytes {
            Cow::Borrowed(s) => std::str::from_utf8(s)
                .map_err(FjallError::Blurhash)
                .map(String::from)
                .map(Arc::from)
                .map(Self),
            Cow::Owned(s) => String::from_utf8(s)
                .map_err(|e| e.utf8_error())
                .map_err(FjallError::Blurhash)
                .map(Arc::from)
                .map(Self),
        }
    }
}

impl<'a> PathNode<'a> for Blurhash {
    const NAME: &'static [u8] = b"blurhash";
}

impl std::fmt::Debug for Blurhash {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        self.0.fmt(f)
    }
}
