use std::borrow::Cow;

use crate::{
    path_gen::{PathItem, PathNode},
    repo::DeleteToken,
};

use super::FjallError;

impl<'a> PathItem<'a> for DeleteToken {
    type Error = FjallError;

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        Self::from_slice(&bytes).ok_or(FjallError::DeleteToken)
    }

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        Cow::Owned(self.to_bytes())
    }
}

impl<'a> PathNode<'a> for DeleteToken {
    const NAME: &'static [u8] = b"delete-token";
}
