use crate::path_gen::{PathItem, PathNode};
use std::borrow::Cow;

use super::FjallError;

pub(super) struct Empty;

impl<'a> PathItem<'a> for Empty {
    type Error = FjallError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        Cow::Borrowed(&[])
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        if bytes.is_empty() {
            Ok(Empty)
        } else {
            Err(FjallError::Empty)
        }
    }
}

impl<'a> PathNode<'a> for Empty {
    const NAME: &'static [u8] = b"empty";
}

impl std::fmt::Debug for Empty {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        f.write_str("<empty>")
    }
}
