use crate::path_gen::{PathItem, PathNode};
use serde_json::Value;
use std::borrow::Cow;

use super::FjallError;

#[derive(Debug)]
pub(super) struct Job(pub Value);

impl<'a> PathItem<'a> for Job {
    type Error = FjallError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        let v = serde_json::to_vec(&self.0).expect("Serialize job");

        Cow::Owned(v)
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        serde_json::from_slice(&bytes)
            .map(Job)
            .map_err(FjallError::Job)
    }
}
impl<'a> PathNode<'a> for Job {
    const NAME: &'static [u8] = b"job";
}
