use std::borrow::Cow;

use crate::{
    path_gen::{PathItem, PathNode},
    repo::JobId,
};

use super::FjallError;

impl<'a> PathItem<'a> for JobId {
    type Error = FjallError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        Cow::Borrowed(self.0.as_bytes())
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        bytes
            .as_ref()
            .try_into()
            .map(JobId::from_bytes)
            .map_err(FjallError::JobId)
    }
}

impl<'a> PathNode<'a> for JobId {
    const NAME: &'static [u8] = b"job-id";
}
