use crate::path_gen::{PathItem, PathNode};
use std::{borrow::Cow, str::Utf8Error};
use url::Url;

#[derive(Debug, Clone)]
pub(super) struct Proxy(pub Url);

#[derive(Debug, thiserror::Error)]
pub(crate) enum ProxyError {
    #[error("Invalid proxy bytes")]
    Utf8(#[from] Utf8Error),

    #[error("Failed parsing proxy url")]
    Url(#[from] url::ParseError),
}

impl<'a> PathItem<'a> for Proxy {
    type Error = ProxyError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        Cow::Borrowed(self.0.as_str().as_bytes())
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        let s = std::str::from_utf8(&bytes)?;

        let url = Url::parse(s)?;

        Ok(Self(url))
    }
}

impl<'a> PathNode<'a> for Proxy {
    const NAME: &'static [u8] = b"proxy";
}
