use crate::path_gen::{PathItem, PathNode};
use std::borrow::Cow;

use super::FjallError;

#[derive(Clone)]
pub(super) struct Queue<'a>(pub Cow<'a, str>);

impl<'a> PathItem<'a> for Queue<'a> {
    type Error = FjallError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        Cow::Borrowed(self.0.as_bytes())
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        match bytes {
            Cow::Borrowed(s) => std::str::from_utf8(s)
                .map(Cow::Borrowed)
                .map(Queue)
                .map_err(FjallError::Queue),
            Cow::Owned(s) => String::from_utf8(s)
                .map_err(|e| e.utf8_error())
                .map(Cow::Owned)
                .map(Queue)
                .map_err(FjallError::Queue),
        }
    }
}

impl<'a> PathNode<'a> for Queue<'a> {
    const NAME: &'static [u8] = b"queue";
}

impl<'a> std::fmt::Debug for Queue<'a> {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        self.0.fmt(f)
    }
}
