use crate::path_gen::{PathItem, PathNode};
use std::borrow::Cow;

use super::FjallError;

#[derive(Debug)]
pub(super) struct RetryCount(pub u64);

impl<'a> PathItem<'a> for RetryCount {
    type Error = FjallError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        Cow::Owned(self.0.to_be_bytes().to_vec())
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        bytes[0..8]
            .try_into()
            .map_err(FjallError::RetryCount)
            .map(u64::from_be_bytes)
            .map(RetryCount)
    }
}

impl<'a> PathNode<'a> for RetryCount {
    const NAME: &'static [u8] = b"retry-count";
}
