use crate::path_gen::{PathItem, PathNode};
use std::borrow::Cow;

use super::FjallError;

#[derive(Clone)]
pub(super) enum UniqueJob {
    New(&'static str),
    Existing(String),
}

impl<'a> PathItem<'a> for UniqueJob {
    type Error = FjallError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        match self {
            Self::New(new) => Cow::Borrowed(new.as_bytes()),
            Self::Existing(existing) => Cow::Borrowed(existing.as_bytes()),
        }
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        match bytes {
            Cow::Borrowed(s) => std::str::from_utf8(s)
                .map_err(FjallError::UniqueJob)
                .map(String::from)
                .map(Self::Existing),
            Cow::Owned(s) => String::from_utf8(s)
                .map_err(|e| e.utf8_error())
                .map_err(FjallError::UniqueJob)
                .map(Self::Existing),
        }
    }
}

impl<'a> PathNode<'a> for UniqueJob {
    const NAME: &'static [u8] = b"unique-job";
}

impl std::fmt::Debug for UniqueJob {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Self::New(s) => s.fmt(f),
            Self::Existing(s) => s.fmt(f),
        }
    }
}
