use std::borrow::Cow;
use uuid::Uuid;

use crate::{
    path_gen::{PathItem, PathNode},
    repo::UploadId,
};

use super::FjallError;

impl<'a> PathItem<'a> for UploadId {
    type Error = FjallError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        Cow::Borrowed(self.as_bytes())
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        Uuid::from_slice(&bytes)
            .map_err(FjallError::UploadId)
            .map(|id| UploadId { id })
    }
}

impl<'a> PathNode<'a> for UploadId {
    const NAME: &'static [u8] = b"upload-id";
}
