use crate::path_gen::{PathItem, PathNode};
use std::borrow::Cow;

use super::{FjallError, InnerUploadResult};

#[derive(Debug)]
pub(super) enum UploadValue {
    Pending,
    Result(InnerUploadResult),
}

impl<'a> PathItem<'a> for UploadValue {
    type Error = FjallError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        match self {
            Self::Pending => Cow::Borrowed(b"1"),
            Self::Result(upload_result) => {
                let bytes = serde_json::to_vec(upload_result).expect("serialize json");
                Cow::Owned(bytes)
            }
        }
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        if &bytes[..] == b"1" {
            Ok(Self::Pending)
        } else {
            serde_json::from_slice(&bytes)
                .map_err(FjallError::Upload)
                .map(Self::Result)
        }
    }
}
impl<'a> PathNode<'a> for UploadValue {
    const NAME: &'static [u8] = b"upload-value";
}
