use crate::path_gen::{PathItem, PathNode};
use std::borrow::Cow;

use super::FjallError;

pub(super) struct Variant(pub String);

impl<'a> PathItem<'a> for Variant {
    type Error = FjallError;

    fn to_bytes(&self) -> Cow<'_, [u8]> {
        Cow::Borrowed(self.0.as_bytes())
    }

    fn parse(bytes: Cow<'a, [u8]>) -> Result<Self, Self::Error>
    where
        Self: Sized + 'a,
    {
        match bytes {
            Cow::Borrowed(bytes) => std::str::from_utf8(bytes)
                .map(String::from)
                .map(Self)
                .map_err(FjallError::Variant),
            Cow::Owned(bytes) => String::from_utf8(bytes)
                .map(Self)
                .map_err(|e| e.utf8_error())
                .map_err(FjallError::Variant),
        }
    }
}

impl<'a> PathNode<'a> for Variant {
    const NAME: &'static [u8] = b"variant";
}

impl std::fmt::Debug for Variant {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        self.0.fmt(f)
    }
}
