#![cfg(system_deps)]

use common::{
    pict_rs_fjall_config, pict_rs_test_config, upload_form, with_pict_rs, OkString, PictRsResult,
    UploadResponse,
};

mod common;

#[derive(serde::Deserialize)]
struct BackgroundResponse {
    #[allow(unused)]
    msg: OkString,

    uploads: Vec<Upload>,
}

#[derive(serde::Deserialize)]
struct Upload {
    upload_id: String,
}

#[test]
fn sled_can_upload_and_download_file() {
    let address = "127.0.0.1:9500";

    let config = pict_rs_test_config(address);

    with_pict_rs(config, || can_upload_and_download_file(address)).unwrap();
}

#[test]
fn fjall_can_upload_and_download_file() {
    let address = "127.0.0.1:9501";

    let config = pict_rs_fjall_config(address);

    with_pict_rs(config, || can_upload_and_download_file(address)).unwrap();
}

async fn can_upload_and_download_file(address: &'static str) {
    let client = reqwest::Client::new();

    let form = upload_form(["./client-examples/earth.gif"]).await;

    let response = client
        .post(format!("http://{address}/image/backgrounded"))
        .multipart(form)
        .send()
        .await
        .expect("send request");

    let backgrounded = response
        .json::<PictRsResult<BackgroundResponse>>()
        .await
        .expect("valid response")
        .unwrap();

    assert_eq!(backgrounded.uploads.len(), 1);

    let upload_id = &backgrounded.uploads[0].upload_id;

    let response = loop {
        let response = client
            .get(format!(
                "http://{address}/image/backgrounded/claim?upload_id={upload_id}"
            ))
            .send()
            .await
            .expect("send request");

        if response.status() == 200 {
            break response;
        }

        assert!(response.status().is_success());
    };

    let upload = response
        .json::<PictRsResult<UploadResponse>>()
        .await
        .expect("valid response")
        .unwrap();

    assert_eq!(upload.files.len(), 1);

    let alias = &upload.files[0].file;

    let response = client
        .get(format!("http://{address}/image/original/{alias}"))
        .send()
        .await
        .expect("download file");

    assert!(response.status().is_success(), "download failed");

    let length = response.bytes().await.expect("downlaod bytes").len();

    assert!(length > 0);
}
